@@ -6,23 +6,23 @@ from typing import List, Type
 
 import click
 
-from memfault_cli.chunk import MemfaultChunk
-from memfault_cli.context import MemfaultCliClickContext
-from memfault_cli.deploy import Deployer
-from memfault_cli.upload import (
+from ._version import version
+from .chunk import MemfaultChunk
+from .context import MemfaultCliClickContext
+from .deploy import Deployer
+from .upload import (
     AndroidAppSymbolsUploader,
     AospSymbolDirectoryUploader,
     AospSymbolUploader,
     BugreportUploader,
     CoredumpUploader,
+    MarUploader,
     McuSdkElfSymbolUploader,
     ReleaseArtifactUploader,
     Uploader,
     XedUploader,
 )
 
-from . import __version__
-
 pass_memfault_cli_context = click.make_pass_decorator(MemfaultCliClickContext, ensure=True)
 click_argument_path = click.argument("path", type=click.Path(exists=True))
 click_option_concurrency = click.option(
@@ -48,13 +48,13 @@ click_option_revision = click.option(
 @click.option("--project", help="Project slug")
 @click.option("--url", hidden=True)
 @click.option("--verbose", help="Log verbosely", is_flag=True)
-@click.version_option(version=__version__)
+@click.version_option(version=version)
 @pass_memfault_cli_context
 def cli(ctx: MemfaultCliClickContext, **kwargs):
     ctx.obj.update(kwargs)
 
     if ctx.verbose:
-        click.echo(f"version: {__version__}")
+        click.echo(f"version: {version}")
 
         logging.basicConfig(level=logging.DEBUG)
         requests_log = logging.getLogger("requests.packages.urllib3")
@@ -67,11 +67,13 @@ def cli(ctx: MemfaultCliClickContext, **kwargs):
         def httpclient_log(*args) -> None:
             httpclient_logger.log(logging.DEBUG, " ".join(args))
 
-        http.client.print = httpclient_log
-        http.client.HTTPConnection.debuglevel = 1
+        http.client.print = httpclient_log  # type: ignore[attr-defined]
+        http.client.HTTPConnection.debuglevel = 1  # type: ignore[attr-defined]
 
 
-def _do_upload_or_raise(ctx: MemfaultCliClickContext, path: str, uploader_cls: Type[Uploader]):
+def _do_upload_or_raise(
+    ctx: MemfaultCliClickContext, path: str, uploader_cls: Type[Uploader]
+) -> None:
     authenticator = ctx.authenticator
     ctx.check_required_cli_args(authenticator=authenticator)
 
@@ -169,17 +171,46 @@ def upload_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs):
     """
     ctx.obj.update(**kwargs)
 
+    uploader: Type[Uploader]
     if os.path.isdir(path):
         uploader = AospSymbolDirectoryUploader
+    elif ctx.software_info is None:
+        uploader = AospSymbolUploader
     else:
-        if ctx.software_info is None:
-            uploader = AospSymbolUploader
-        else:
-            uploader = McuSdkElfSymbolUploader
+        uploader = McuSdkElfSymbolUploader
 
     _do_upload_or_raise(ctx, path, uploader)
 
 
+@cli.command(name="upload-mar")
+@click.option(
+    "--hardware-version",
+    required=True,
+    help="Required to identify the type of Android hardware, see https://mflt.io/34PyNGQ/#android. By default the ro.product.board property is used.",
+)
+@click.option(
+    "--software-type",
+    required=True,
+    help="Required to identify the Android system software, see https://mflt.io/34PyNGQ/#android-1",
+)
+@click.option(
+    "--software-version",
+    required=True,
+    help="Required to identify single builds on Android devices, see https://mflt.io/34PyNGQ/#software-version",
+)
+@click.option(
+    "--device-serial",
+    required=True,
+    help="The unique identifier of a device",
+)
+@click_argument_path
+@pass_memfault_cli_context
+def upload_mar(ctx: MemfaultCliClickContext, path: str, **kwargs):
+    """Upload a Memfault Archive File (mar) file for analysis."""
+    ctx.obj.update(**kwargs)
+    _do_upload_or_raise(ctx, path, MarUploader)
+
+
 @cli.command(name="upload-mcu-symbols")
 @click.option(
     "--software-type",
@@ -253,9 +284,13 @@ def upload_aosp_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs):
     """
     ctx.obj.update(**kwargs)
 
-    _do_upload_or_raise(
-        ctx, path, AospSymbolDirectoryUploader if os.path.isdir(path) else AospSymbolUploader
-    )
+    uploader: Type[Uploader]
+    if os.path.isdir(path):
+        uploader = AospSymbolDirectoryUploader
+    else:
+        uploader = AospSymbolUploader
+
+    _do_upload_or_raise(ctx, path, uploader)
 
 
 @cli.command(name="upload-android-app-symbols")
@@ -302,7 +337,8 @@ def upload_android_app_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs
     the required information directly.
     """
     ctx.obj.update(**kwargs)
-    _do_upload_or_raise(ctx, path, AndroidAppSymbolsUploader)
+    uploader: Type[Uploader] = AndroidAppSymbolsUploader
+    _do_upload_or_raise(ctx, path, uploader)
 
 
 @cli.command(name="upload-ota-payload")
@@ -316,12 +352,12 @@ def upload_android_app_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs
 @click.option(
     "--delta-from",
     required=False,
-    help="Pass --delta-from FROM_VERSION make the OTA Payload a Delta Release payload from the version FROM_VERSION. Use alongside --delta-target TARGET_VERSION",
+    help="Pass --delta-from FROM_VERSION make the OTA Payload a Delta Release payload from the version FROM_VERSION. Use alongside --delta-to TO_VERSION",
 )
 @click.option(
-    "--delta-target",
+    "--delta-to",
     required=False,
-    help="Pass --delta-target TARGET_VERSION to make the OTA Payload a Delta Release payload target version TARGET_VERSION. Use alongside --delta-from FROM_VERSION.",
+    help="Pass --delta-to TO_VERSION to make the OTA Payload a Delta Release payload to version TO_VERSION. Use alongside --delta-from FROM_VERSION.",
 )
 @click.option("--notes", default="", help="Optional release notes.")
 @click.option(
@@ -363,7 +399,7 @@ def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
             --hardware-version mp \\
             --software-type stm32-fw \\
             --delta-from 0.9.0 \\
-            --delta-target 1.0.0-alpha \\
+            --delta-to 1.0.0-alpha \\
             --revision 89335ffade90ff7697e2ce5238bd4c68978b6d6e \\
             build/stm32-fw-delta.bin
 
@@ -371,7 +407,7 @@ def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
     """
     ctx.obj.update(**kwargs)
     ctx.check_required_either(
-        {"software_version"}, {"delta_from", "delta_target"}, mutually_exclusive=True
+        {"software_version"}, {"delta_from", "delta_to"}, mutually_exclusive=True
     )
     _do_upload_or_raise(ctx, path, ReleaseArtifactUploader)
 
@@ -379,7 +415,7 @@ def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
 @cli.command(name="deploy-release")
 @click.option("--release-version", type=str, required=False)
 @click.option("--delta-from", required=False)
-@click.option("--delta-target", required=False)
+@click.option("--delta-to", required=False)
 @click.option("--cohort", type=str, required=True)
 @click.option(
     "--rollout-percent",
@@ -405,11 +441,11 @@ def deploy_release(ctx: MemfaultCliClickContext, cohort: str, rollout_percent: i
 
     ctx.check_required_cli_args(authenticator=ctx.authenticator)
     ctx.check_required_either(
-        {"release_version"}, {"delta_from", "delta_target"}, mutually_exclusive=True
+        {"release_version"}, {"delta_from", "delta_to"}, mutually_exclusive=True
     )
     release_version = ctx.obj.get("release_version") or (
-        ctx.obj.get("delta_from"),
-        ctx.obj.get("delta_target"),
+        ctx.obj["delta_from"],
+        ctx.obj["delta_to"],
     )
     deployer = Deployer(ctx=ctx, authenticator=ctx.authenticator)
     deployer.deploy(cohort=cohort, release_version=release_version, rollout_percent=rollout_percent)
@@ -461,7 +497,7 @@ def post_chunk(ctx: MemfaultCliClickContext, encoding, data, **kwargs):
 
     ctx.obj.update(**kwargs)
     if encoding == "hex":
-        chunks = [bytearray.fromhex(data)]
+        chunks = [bytes.fromhex(data)]
     elif encoding == "base64":
         chunks = [b64decode(data)]
     elif encoding == "bin":
