@@ -1,11 +1,11 @@
 import json
 import logging
-from typing import Iterator, List, Optional, Set, Union
+from typing import Any, Dict, List, Optional, Set, Union
 
 import click
 
-from memfault_cli.authenticator import Authenticator
-from memfault_cli.functools_ext import cached_property
+from .authenticator import Authenticator
+from .functools_ext import cached_property
 
 LOG = logging.getLogger(__name__)
 
@@ -26,12 +26,12 @@ class FullReleaseVersion:  # noqa: B903 - no data classes in Python 3.6
 
 
 class DeltaReleaseVersion:  # noqa: B903 - no data classes in Python 3.6
-    def __init__(self, delta_from: str, delta_target: str):
+    def __init__(self, delta_from: str, delta_to: str):
         self.delta_from = delta_from
-        self.delta_target = delta_target
+        self.delta_to = delta_to
 
     def to_json_representable(self):
-        return [self.delta_from, self.delta_target]
+        return [self.delta_from, self.delta_to]
 
 
 class SoftwareInfo:  # noqa: B903 - no data classes in Python 3.6
@@ -46,7 +46,7 @@ class SoftwareInfo:  # noqa: B903 - no data classes in Python 3.6
         self.revision = revision
 
 
-CustomMetricReadingValueType = Union[int, float, str, bool]
+CustomMetricReadingValueType = Union[int, float, str, bool, None]
 
 
 class Attribute:  # noqa: B903 - no data classes in Python 3.6
@@ -80,6 +80,16 @@ class XedInfo:  # noqa: B903 - no data classes in Python 3.6
         self.hardware_version = hardware_version
 
 
+class MarInfo:  # noqa: B903 - no data classes in Python 3.6
+    def __init__(
+        self, device_serial: str, hardware_version: str, software_type: str, software_version: str
+    ):
+        self.device_serial = device_serial
+        self.hardware_version = hardware_version
+        self.software_type = software_type
+        self.software_version = software_version
+
+
 class MemfaultCliClickContext(object):
     """
     A context passed around between the memfault cli sub-commands.
@@ -92,7 +102,7 @@ class MemfaultCliClickContext(object):
     """
 
     def __init__(self) -> None:
-        self.obj = {}
+        self.obj: Dict[str, Any] = {}
 
     def _format_as_option(self, property_name: str) -> str:
         return f"--{property_name.replace('_', '-')}"
@@ -109,11 +119,11 @@ class MemfaultCliClickContext(object):
         for arg in required_args:
             self._find_obj_or_raise(arg)
 
-    def _format_arg_set(self, arg_set: Iterator[Set[str]]) -> str:
+    def _format_arg_set(self, arg_set: Set[str]) -> str:
         return ", ".join([f"{self._format_as_option(arg)!r}" for arg in sorted(arg_set)])
 
     def check_required_either(self, *args: Set[str], mutually_exclusive=False):
-        """ Check that any full set of args is present """
+        """Check that any full set of args is present"""
         valid_arg_sets = []
         passed_arg_sets = []
 
@@ -174,17 +184,19 @@ class MemfaultCliClickContext(object):
         sw_ver = self.obj.get("software_version")
         revision = self.obj.get("revision")
         delta_from = self.obj.get("delta_from")
-        delta_target = self.obj.get("delta_target")
+        delta_to = self.obj.get("delta_to")
 
-        version = None
-        if delta_from and delta_target:
-            version = DeltaReleaseVersion(delta_from, delta_target)
+        version: Union[DeltaReleaseVersion, FullReleaseVersion, None]
+        if delta_from and delta_to:
+            version = DeltaReleaseVersion(delta_from, delta_to)
         elif sw_ver:
             version = FullReleaseVersion(sw_ver)
+        else:
+            version = None
 
         if revision and (version is None or sw_type is None):
             raise click.exceptions.UsageError(
-                "A version ('--software-version' or '--delta-{{from,target}}') and '--software-type' must be specified when using '--revision'"
+                "A version ('--software-version' or '--delta-{{from,to}}') and '--software-type' must be specified when using '--revision'"
             )
 
         if sw_type is None and version is None:
@@ -192,7 +204,7 @@ class MemfaultCliClickContext(object):
 
         if sw_type is None or version is None:
             raise click.exceptions.UsageError(
-                "Version ('--software-version' or '--delta-{{from,target}}') and '--software-type' must be specified together"
+                "Version ('--software-version' or '--delta-{{from,to}}') and '--software-type' must be specified together"
             )
 
         return SoftwareInfo(
@@ -214,6 +226,19 @@ class MemfaultCliClickContext(object):
             hardware_version=hardware_version,
         )
 
+    @property
+    def mar_info(self) -> MarInfo:
+        device_serial = self._find_obj_or_raise("device_serial")
+        hardware_version = self._find_obj_or_raise("hardware_version")
+        software_type = self._find_obj_or_raise("software_type")
+        software_version = self._find_obj_or_raise("software_version")
+        return MarInfo(
+            device_serial=device_serial,
+            hardware_version=hardware_version,
+            software_type=software_type,
+            software_version=software_version,
+        )
+
     @property
     def file_url(self) -> str:
         url = self.obj.get("url")
