@@ -1,9 +1,10 @@
-from typing import Container, Iterable, Optional
+from typing import Iterable, Optional
 
+import more_itertools
+from elftools.construct import Container
 from elftools.elf.elffile import ELFFile
 from elftools.elf.sections import NoteSection
 from elftools.elf.segments import NoteSegment
-from more_itertools import first
 
 # FIXME: Deduplicate. Copy pasta from elf_utils.py
 
@@ -26,7 +27,9 @@ def get_notes(elf: ELFFile) -> Iterable[Container]:
 
 
 def get_gnu_build_id(elf: ELFFile) -> Optional[str]:
-    build_id_note = first(filter(lambda note: note.n_type == NT_GNU_BUILD_ID, get_notes(elf)), None)
+    build_id_note = more_itertools.first_true(
+        get_notes(elf), pred=lambda note: note.n_type == NT_GNU_BUILD_ID
+    )
     if not build_id_note:
         return None
     return build_id_note.n_desc
