@@ -1,8 +1,9 @@
+import abc
 import hashlib
 import logging
 import os
 import re
-import urllib
+import urllib.parse
 from abc import abstractmethod
 from concurrent.futures import ThreadPoolExecutor
 from glob import glob
@@ -19,9 +20,9 @@ from tqdm import tqdm
 from tqdm.utils import CallbackIOWrapper
 from urllib3 import Retry
 
-from . import functools_ext
 from .authenticator import Authenticator
 from .context import DeltaReleaseVersion, MemfaultCliClickContext, PackageInfo
+from .functools_ext import cached_property
 from .gnu_build_id import get_gnu_build_id
 
 LOG = logging.getLogger(__name__)
@@ -57,7 +58,7 @@ def url_quote(value: str) -> str:
     return urllib.parse.quote(value, safe="", encoding="utf-8")
 
 
-class Uploader:
+class Uploader(metaclass=abc.ABCMeta):
     def __init__(
         self, *, ctx: MemfaultCliClickContext, file_path: str, authenticator: Authenticator
     ):
@@ -72,7 +73,7 @@ class Uploader:
             total=5,
             backoff_factor=2,  # Sleep for 2s, 4s, 8s, 16s, 32s, Stop
             status_forcelist=[429, 500, 502, 503, 504],
-            method_whitelist=["HEAD", "GET", "POST", "PUT"],
+            allowed_methods=["HEAD", "GET", "POST", "PUT"],
         )
         adapter = HTTPAdapter(max_retries=retry_strategy)
         session = requests.Session()
@@ -160,24 +161,24 @@ class Uploader:
         )
         # This file had already uploaded (maybe concurrently). Gracefully ignore the error then.
         if response.status_code == 409:
-            LOG.info(f"{self.file_path}: was already uploaded.")
+            LOG.info("%s: was already uploaded.", self.file_path)
             return
 
         check_response(response)
 
     def upload(self, *, progressbar=True) -> bool:
         if self._is_already_uploaded():
-            LOG.info(f"{self.file_path}: skipping, already uploaded.")
+            LOG.info("%s: skipping, already uploaded.", self.file_path)
             return False
         upload_url, token = self._prepare_upload()
         self._put_file(upload_url, progressbar=progressbar)
         self._post_token(token)
-        LOG.info(f"{self.file_path}: uploaded!")
+        LOG.info("%s: uploaded!", self.file_path)
         if self.ui_url():
             click.echo(f"You can view in the UI here:\n   {self.ui_url()}")
         return True
 
-    @functools_ext.cached_property
+    @cached_property
     def file_md5(self):
         with open(self.file_path, "rb") as f:
             return hash_io_hexdigest("md5", f)
@@ -190,7 +191,7 @@ class Uploader:
 class BugreportUploader(Uploader):
     def can_upload_file(self) -> bool:
         if not is_zipfile(self.file_path):
-            LOG.error(f"{self.file_path} is not a valid zip file!")
+            LOG.error("%s is not a valid zip file!", self.file_path)
             return False
         return True
 
@@ -206,7 +207,7 @@ class CoredumpUploader(Uploader):
         with open(self.file_path, "rb") as f:
             hdr = f.read(4).decode("ascii", errors="ignore")
             if hdr != "CORE":
-                LOG.error(f"{self.file_path} is not a Memfault Coredump")
+                LOG.error("%s is not a Memfault Coredump", self.file_path)
                 return False
 
         return True
@@ -239,6 +240,29 @@ class XedUploader(Uploader):
         }
 
 
+class MarUploader(Uploader):
+    def can_upload_file(self) -> bool:
+        if not is_zipfile(self.file_path):
+            LOG.error(f"{self.file_path} is not a valid zip file!")
+            return False
+        return True
+
+    def entity_url(self) -> str:
+        if self.authenticator.project_key_auth():
+            return f"{self._api_base_url()}/upload/mar"
+        else:
+            return f"{self._projects_base_url()}/mar"
+
+    def post_token_extra_request_data(self) -> Optional[Dict]:
+        info = self.ctx.mar_info
+        return {
+            "device_serial": info.device_serial,
+            "hardware_version": info.hardware_version,
+            "software_type": info.software_type,
+            "software_version": info.software_version,
+        }
+
+
 class AospSymbolUploader(Uploader):
     gnu_build_id: str = ""
 
@@ -254,20 +278,21 @@ class AospSymbolUploader(Uploader):
             elf = ELFFile(f)
             return get_gnu_build_id(elf), elf.has_dwarf_info()
 
-    def can_upload_file(self) -> Optional[Any]:
+    def can_upload_file(self) -> bool:
         if not self._is_elf():
-            LOG.info(f"{self.file_path}: Not an ELF file")
+            LOG.info("%s: Not an ELF file", self.file_path)
             return False
 
-        self.gnu_build_id, has_debug_info = self._get_gnu_build_id_and_has_debug_info()
-        if not self.gnu_build_id:
-            LOG.info(f"{self.file_path}: looks like an ELF but does not contain a GNU Build ID")
+        gnu_build_id, has_debug_info = self._get_gnu_build_id_and_has_debug_info()
+        if not gnu_build_id:
+            LOG.info("%s: looks like an ELF but does not contain a GNU Build ID", self.file_path)
             return False
+        self.gnu_build_id = gnu_build_id
         if not has_debug_info:
-            LOG.info(f"{self.file_path}: looks like an ELF but it has no .debug_info")
+            LOG.info("%s: looks like an ELF but it has no .debug_info", self.file_path)
             return False
         LOG.info(
-            f"{self.file_path}: ELF file with .debug_info and GNU Build ID: {self.gnu_build_id}"
+            "%s: ELF file with .debug_info and GNU Build ID: %s", self.file_path, self.gnu_build_id
         )
         return True
 
@@ -292,25 +317,26 @@ class McuSdkElfSymbolUploader(Uploader):
     def can_upload_file(self) -> bool:
         with open(self.file_path, "rb") as file:
             if file.read(4) != b"\x7FELF":
-                LOG.info(f"{self.file_path}: Not an ELF file")
+                LOG.info("%s: Not an ELF file", self.file_path)
                 return False
             file.seek(0)
 
             inspector = BuildIdInspectorAndPatcher(file)
             if not inspector.elf.has_dwarf_info():
-                LOG.info(f"{self.file_path}: looks like an ELF but it has no .debug_info")
+                LOG.info("%s: looks like an ELF but it has no .debug_info", self.file_path)
                 return False
 
             mcu_build_id_type, mcu_build_id, mcu_build_id_short_len = inspector.get_build_info()
 
             if not self.ctx.software_info and mcu_build_id is None:
                 LOG.error(
-                    f"{self.file_path}: Build Id missing. Specify --software-version and --software-type options "
-                    f"or add a Build Id (see https://mflt.io/symbol-file-build-ids)"
+                    "%s: Build Id missing. Specify --software-version and --software-type options "
+                    "or add a Build Id (see https://mflt.io/symbol-file-build-ids)",
+                    self.file_path,
                 )
                 return False
 
-        LOG.info(f"{self.file_path}: ELF file with .debug_info. Build Id: {mcu_build_id}")
+        LOG.info("%s: ELF file with .debug_info. Build Id: %s", self.file_path, mcu_build_id)
         return True
 
     def _is_already_uploaded(self) -> bool:
@@ -347,7 +373,7 @@ class McuSdkElfSymbolUploader(Uploader):
 
 
 class ReleaseArtifactUploader(Uploader):
-    def can_upload_file(self) -> Optional[Any]:
+    def can_upload_file(self) -> bool:
         # There are no restrictions for the type of binary a customer uses for
         # a release
         return True
@@ -366,23 +392,26 @@ class ReleaseArtifactUploader(Uploader):
         if not info:
             return releases_url
 
-        is_delta = isinstance(info.version, DeltaReleaseVersion)
-
         if info.software_type == "android-build":
             # In case of Android, the created Release.version may not be the same
             # as the SoftwareVersion.version, in case build fingerprints are used,
             # the Release.version will be "stripped" (i.e. */*/*:*/*/*:*/*::1.0.0):
-            return delta_releases_url if is_delta else releases_url
+            return (
+                delta_releases_url
+                if isinstance(info.version, DeltaReleaseVersion)
+                else releases_url
+            )
 
-        if is_delta:
+        if isinstance(info.version, DeltaReleaseVersion):
             from_version = url_quote(info.version.delta_from)
-            target_version = url_quote(info.version.delta_target)
-            return f"{delta_releases_url}/{from_version}/{target_version}"
+            to_version = url_quote(info.version.delta_to)
+            return f"{delta_releases_url}/{from_version}/{to_version}"
 
         return f"{releases_url}/{url_quote(info.version.software_version)}"
 
     def post_token_extra_request_data(self) -> Optional[Dict]:
         info = self.ctx.software_info
+        assert info  # mypy
 
         return {
             "software_version": {
@@ -397,7 +426,7 @@ class ReleaseArtifactUploader(Uploader):
 
 
 class DirectoryUploader(Uploader):
-    uploader_cls: Type[Uploader] = None
+    uploader_cls: Type[Uploader]
 
     def can_upload_file(self) -> bool:
         return os.path.isdir(self.file_path)
@@ -418,7 +447,7 @@ class DirectoryUploader(Uploader):
                 queued_uploaders.append(uploader)
                 did_upload |= True
             else:
-                LOG.info(f"{file_path}: skipping...")
+                LOG.info("%s: skipping...", file_path)
 
         def do_upload(uploader: Uploader):
             uploader.upload(progressbar=False)
@@ -453,7 +482,7 @@ class ProguardMappingUploader(Uploader):
         super().__init__(ctx=ctx, file_path=file_path, authenticator=authenticator)
         self.package_info = package_info
 
-    def can_upload_file(self) -> Optional[Any]:
+    def can_upload_file(self) -> bool:
         return True
 
     def entity_url(self) -> str:
@@ -472,7 +501,8 @@ class AndroidAppSymbolsUploader(Uploader):
     def can_upload_file(self) -> bool:
         if not os.path.isdir(self.file_path):
             LOG.error(
-                f'{self.file_path} is not a directory. Please specify the Android app\'s root "build" path instead!'
+                '%s is not a directory. Please specify the Android app\'s root "build" path instead!',
+                self.file_path,
             )
             return False
         return True
@@ -548,3 +578,6 @@ class AndroidAppSymbolsUploader(Uploader):
         if not did_upload:
             raise click.exceptions.UsageError("No files uploaded!")
         return did_upload
+
+    def entity_url(self) -> str:
+        raise NotImplementedError
